/*
 Develop a program to maintain a database of
 records in order to track/store people's
 names, addresses and phone numbers.

  Author: Me
  Date:   Today
*/

#include <stdio.h>
#include <stdlib.h>
#include <search.h>
#include <string.h>
#include <ctype.h>

/* Contact
   -------
   Last Name
   First Name
   Address 1
   Address 2
   City
   State
   Zip
   Phone Number (w/Area Code)
*/

typedef struct Contact
{
	char	first_name[31];
	char	last_name[31];
	char	address_1[31];
	char	address_2[31];
	char	city[31];
	char	state[3];
	char	zip[11];
	char	phone[21];
} CONTACT;

CONTACT *pRecords;
int      nRecords = 0;

/* Major Functions
   ---------------
   Edit Records
   Delete Record
   List Records
   Print Records
   Search
   Sort
*/

/*****************************************
 * Instructor's Note -
 *
 * For simplicity, I have copied the functions
 * we created for the "MyLib" sample here, so
 * I don't need to worry about adding the library
 * to the project.
 *
 * Recreating the library and using it instead
 * of copying the functions here is left as an
 * exercise for the students.
 *****************************************/

char repeat()
{
	char	answer;

	do
	{
		printf( "\nContinue? (Y/N)" );
		
		answer = tolower(fgetc(stdin));
		fflush(stdin);

		if( answer == 'y' || answer == 'n' )
			break;
		else
			printf( "\a\nPlease enter Y)es or N)o only!" );
	}
	while( 1 );

	return answer;
}

void input_string( const char *prompt, char *pData, int maxlen )
{
	char	buff[100];

	printf( "Enter %s [%s]: ", prompt, pData );
	fflush(stdin);
	gets( buff );

	if( strlen(buff) )
	{
		strncpy( pData, buff, maxlen );
		*(pData+maxlen) = '\0';
	}
}

void input_integer( const char *prompt, int *pData )
{
	char	tmpstr[20];

	sprintf( tmpstr, "%d", *pData );
	input_string( prompt, tmpstr, 19 );
	sscanf( tmpstr, "%d", pData );
}

void input_float( const char *prompt, float *pData )
{
	char	tmpstr[20];

	sprintf( tmpstr, "%f", *pData );
	input_string( prompt, tmpstr, 19 );
	sscanf( tmpstr, "%f", pData );
}

void input_double( const char *prompt, double *pData )
{
	char	tmpstr[20];

	sprintf( tmpstr, "%lf", *pData );
	input_string( prompt, tmpstr, 19 );
	sscanf( tmpstr, "%lf", pData );
}


/******************************************
 * sortname
 *
 * This function is used by the 'qsort' function
 * to compare 2 CONTACT records to determine if
 * they are less than or greater than the other.
 *
 * The qsort function will call this function as
 * needed and we must compare the field from the
 * record and return -1 (<), 0 (==), or 1 (>).
 ******************************************/

int sortname( const void *pRec1, const void *pRec2 )
{
	//
	// Since qsort does not know that it is
	// sorting CONTACT records, it gives us
	// a void (generic) pointer.  Cast it to
	// our data type pointer so we can get
	// to the members in the structure.
	//
	const CONTACT	*pContact1 = (const CONTACT*) pRec1;
	const CONTACT	*pContact2 = (const CONTACT*) pRec2;

	int	result;

	//
	// The stricmp compares two strings (ignoring case)
	// and return -1 (less), 0 (equal), or 1 (greater)
	//
	result = stricmp( pContact1->last_name, pContact2->last_name );

	if( result == 0 )
	{
		//
		// If the last names were equal, check
		// the first names also
		//
		result = stricmp( pContact1->first_name, pContact2->first_name );
	}

	return result;
}

/******************************************
 * sortcity
 *
 * This function is used by the 'qsort' function
 * to compare 2 CONTACT records to determine if
 * they are less than or greater than the other.
 *
 * The qsort function will call this function as
 * needed and we must compare the field from the
 * record and return -1 (<), 0 (==), or 1 (>).
 ******************************************/

int sortcity( const void *pRec1, const void *pRec2 )
{
	//
	// Since qsort does not know that it is
	// sorting CONTACT records, it gives us
	// a void (generic) pointer.  Cast it to
	// our data type pointer so we can get
	// to the members in the structure.
	//
	const CONTACT	*pContact1 = (const CONTACT*) pRec1;
	const CONTACT	*pContact2 = (const CONTACT*) pRec2;

	int	result;

	//
	// The stricmp compares two strings (ignoring case)
	// and return -1 (less), 0 (equal), or 1 (greater)
	//
	result = stricmp( pContact1->city, pContact2->city );

	if( result == 0 )
	{
		//
		// If the city's were equal, check
		// the last and first names also
		//
		result = sortname(pRec1, pRec2 );
	}

	return result;
}

/******************************************
 * sortstate
 *
 * This function is used by the 'qsort' function
 * to compare 2 CONTACT records to determine if
 * they are less than or greater than the other.
 *
 * The qsort function will call this function as
 * needed and we must compare the field from the
 * record and return -1 (<), 0 (==), or 1 (>).
 ******************************************/

int sortstate( const void *pRec1, const void *pRec2 )
{
	//
	// Since qsort does not know that it is
	// sorting CONTACT records, it gives us
	// a void (generic) pointer.  Cast it to
	// our data type pointer so we can get
	// to the members in the structure.
	//
	const CONTACT	*pContact1 = (const CONTACT*) pRec1;
	const CONTACT	*pContact2 = (const CONTACT*) pRec2;

	int	result;

	//
	// The stricmp compares two strings (ignoring case)
	// and return -1 (less), 0 (equal), or 1 (greater)
	//
	result = stricmp( pContact1->state, pContact2->state );

	if( result == 0 )
	{
		//
		// If the state's were equal, check
		// the city and names also
		//
		result = sortcity(pRec1, pRec2 );
	}

	return result;
}

/******************************************
 * sortzip
 *
 * This function is used by the 'qsort' function
 * to compare 2 CONTACT records to determine if
 * they are less than or greater than the other.
 *
 * The qsort function will call this function as
 * needed and we must compare the field from the
 * record and return -1 (<), 0 (==), or 1 (>).
 ******************************************/

int sortzip( const void *pRec1, const void *pRec2 )
{
	//
	// Since qsort does not know that it is
	// sorting CONTACT records, it gives us
	// a void (generic) pointer.  Cast it to
	// our data type pointer so we can get
	// to the members in the structure.
	//
	const CONTACT	*pContact1 = (const CONTACT*) pRec1;
	const CONTACT	*pContact2 = (const CONTACT*) pRec2;

	int	result;

	//
	// The stricmp compares two strings (ignoring case)
	// and return -1 (less), 0 (equal), or 1 (greater)
	//
	result = stricmp( pContact1->zip, pContact2->zip );

	if( result == 0 )
	{
		//
		// If the zip codes were equal, check
		// the last and first names also
		//
		result = sortname( pRec1, pRec2 );
	}

	return result;
}

/******************************************
 * sortphone
 *
 * This function is used by the 'qsort' function
 * to compare 2 CONTACT records to determine if
 * they are less than or greater than the other.
 *
 * The qsort function will call this function as
 * needed and we must compare the field from the
 * record and return -1 (<), 0 (==), or 1 (>).
 ******************************************/

int sortphone( const void *pRec1, const void *pRec2 )
{
	//
	// Since qsort does not know that it is
	// sorting CONTACT records, it gives us
	// a void (generic) pointer.  Cast it to
	// our data type pointer so we can get
	// to the members in the structure.
	//
	const CONTACT	*pContact1 = (const CONTACT*) pRec1;
	const CONTACT	*pContact2 = (const CONTACT*) pRec2;

	int	result;

	//
	// The stricmp compares two strings (ignoring case)
	// and return -1 (less), 0 (equal), or 1 (greater)
	//
	result = stricmp( pContact1->phone, pContact2->phone );

	return result;
}

/******************************************
 * searchname
 *
 * This function is used by the 'bsearch' function
 * to compare a CONTACT record to the key being
 * searched.  
 *
 ******************************************/

int searchname( const void *pRec1, const void *pRec2 )
{
	const char		*pKey = (const char*) pRec1;
	const CONTACT	*pContact = (const CONTACT*) pRec2;

	int	result;

	//
	// The stricmp compares two strings (ignoring case)
	// and return -1 (less), 0 (equal), or 1 (greater)
	//
	result = stricmp( pKey, pContact->last_name );

	return result;
}

/******************************************
 * searchcity
 *
 * This function is used by the 'bsearch' function
 * to compare a CONTACT record to the key being
 * searched.  
 *
 ******************************************/

int searchcity( const void *pRec1, const void *pRec2 )
{
	const char		*pKey = (const char*) pRec1;
	const CONTACT	*pContact = (const CONTACT*) pRec2;

	int	result;

	//
	// The stricmp compares two strings (ignoring case)
	// and return -1 (less), 0 (equal), or 1 (greater)
	//
	result = stricmp( pKey, pContact->city );

	return result;
}

/******************************************
 * searchstate
 *
 * This function is used by the 'bsearch' function
 * to compare a CONTACT record to the key being
 * searched.  
 *
 ******************************************/

int searchstate( const void *pRec1, const void *pRec2 )
{
	const char		*pKey = (const char*) pRec1;
	const CONTACT	*pContact = (const CONTACT*) pRec2;

	int	result;

	//
	// The stricmp compares two strings (ignoring case)
	// and return -1 (less), 0 (equal), or 1 (greater)
	//
	result = stricmp( pKey, pContact->state );

	return result;
}

/******************************************
 * searchzip
 *
 * This function is used by the 'bsearch' function
 * to compare a CONTACT record to the key being
 * searched.  
 *
 ******************************************/

int searchzip( const void *pRec1, const void *pRec2 )
{
	const char		*pKey = (const char*) pRec1;
	const CONTACT	*pContact = (const CONTACT*) pRec2;

	int	result;

	//
	// The stricmp compares two strings (ignoring case)
	// and return -1 (less), 0 (equal), or 1 (greater)
	//
	result = stricmp( pKey, pContact->zip );

	return result;
}

/******************************************
 * searchphone
 *
 * This function is used by the 'bsearch' function
 * to compare a CONTACT record to the key being
 * searched.  
 *
 ******************************************/

int searchphone( const void *pRec1, const void *pRec2 )
{
	const char		*pKey = (const char*) pRec1;
	const CONTACT	*pContact = (const CONTACT*) pRec2;

	int	result;

	//
	// The stricmp compares two strings (ignoring case)
	// and return -1 (less), 0 (equal), or 1 (greater)
	//
	result = stricmp( pKey, pContact->phone );

	return result;
}

void clearscreen()
{
	// NOTE: For unix, try 'system( "clear" );' instead
	system( "CLS" );
}

/*
 ReadRecords
*/

void ReadRecords()
{
	FILE	*fp;
	long	filesize;

	fp = fopen( "CONTACT.DB", "rb" );
	if( fp != NULL )
	{
		fseek( fp, 0, SEEK_END );

		filesize = ftell(fp);

		pRecords = (CONTACT*) malloc( filesize );

		if( pRecords != NULL )
		{
			nRecords = filesize / sizeof(CONTACT);

			rewind(fp);

			fread( pRecords, sizeof(CONTACT), nRecords, fp );

		}
		else
		{
			nRecords = 0;
		}

		fclose(fp);
	}
}

/*
  WriteRecords
*/

void WriteRecords()
{
	FILE	*fp;

	if( nRecords > 0 )
	{
		fp = fopen( "CONTACT.DB", "wb" );

		if( fp != NULL )
		{
			fwrite( pRecords, sizeof(CONTACT), nRecords, fp );
			fclose(fp);
		}
		else
			fprintf( stderr, "ERROR OPENING/CREATING CONTACT.DB!" );
	}
	else
		unlink( "CONTACT.DB" );
}

void edit_contact( CONTACT *pRec )
{
	input_string( "First Name", pRec->first_name, 30 );
	input_string( "Last Name", pRec->last_name, 30 );
	input_string( "Address 1", pRec->address_1, 30 );
	input_string( "Address 2", pRec->address_2, 30 );
	input_string( "City", pRec->city, 30 );
	input_string( "State (e.g. AL)", pRec->state, 2 );
	input_string( "Zip", pRec->zip, 10 );
	input_string( "Phone Number", pRec->phone, 20 );
}

int SelectRecord()
{
	int		iRecord = 1;
	char	prompt[80];

	sprintf( prompt, "Enter record number 1-%d (-1 to cancel) : ", nRecords);

	do
	{
		input_integer( prompt, &iRecord );

		if( iRecord >= 1 && iRecord <= nRecords )
			return iRecord - 1;

		if( iRecord == -1 )
			return iRecord;

		printf( "\n\aError: Out of range.  Try again.\n\n" );
	}
	while( 1 );
}


void AddRecord()
{
	// Step 1: Expand buffer by 1 CONTACT
	pRecords = (CONTACT*) realloc( pRecords, sizeof(CONTACT) * (nRecords + 1) );
	
	if( pRecords == NULL )
	{
		fprintf( stderr, "ERROR:  Out of memory!" );
		exit(2);
	}

	// Step 2: Let user edit fields

	memset( pRecords+nRecords, 0, sizeof(CONTACT) );

	edit_contact( pRecords + nRecords );

	// Step 3: Add 1 to nRecords
	++nRecords;
}

void EditRecord()
{
	int		iRecord;

	clearscreen();

	// Step 1: Let user select record to edit
	iRecord = SelectRecord();

	// If user enters -1, abort edit
	if( iRecord == -1 )
		return;

	// Step 2: Let user edit the selected record
	edit_contact( pRecords + iRecord );
}

void DeleteRecord()
{
	int		iRecord, iNext;

	clearscreen();

	// Step 1: Let user select record to delete
	iRecord = SelectRecord();

	// If user enters -1, abort delete
	if( iRecord == -1 )
		return;

	// Step 2: Setup a loop to shift records up one spot
	for( iNext = iRecord; iNext < nRecords - 1; iNext++ )
	{
		memcpy( pRecords + iNext, pRecords + iNext + 1, sizeof(CONTACT) );
	}

	// Step 3: Resize the array and subtract one record
	pRecords = realloc( pRecords, sizeof(CONTACT) * --nRecords );
}

void DisplayRecords()
{
	CONTACT	*pNext = pRecords;		// Pointer for looping

	int		iRecord = 1,			// Counters
			iPage = 1;
	
	char	name[60],				// Temp String Buffers
			address[61],
			citystatezip[43];

	//
	// Step 1: Prepare title of screen
	//
	clearscreen();

	printf( "                                CONTACTS             Page: %d\n\n", iPage );
	
	printf( "### NAME                 ADDRESS              CITY, ST ZIP    PHONE NUMBER\n" );
	printf( "--- -------------------- -------------------- --------------- ----------------\n" );


	//
	// Step 2: Loop through all records
	//
	do
	{
		//
		// Step 3: Pause after each screenful
		//
		if( iRecord % 21 == 0 )		// Use % for determinine where to break
		{
			//
			// Instructor's Note -
			//
			// Remember that you can declare more variables
			// anytime you begin a new block of code
			//
			char	key;

			printf( "\nPress enter to continue..." );
			fflush(stdin);
			scanf( "%c", &key );

			clearscreen();

			printf( "                                CONTACTS             Page: %d\n\n", iPage++ );
			
			printf( "### NAME                 ADDRESS              CITY, ST ZIP    PHONE NUMBER\n" );
			printf( "--- -------------------- -------------------- --------------- ----------------\n" );

		} // <----- Local variable 'key' drops out of scope here

		//
		// Use sprintf to 'format' to a string inside of stdout
		//
		sprintf( name, "%s, %s", pNext->last_name, pNext->first_name );

		//
		// Format the address also.
		//
		strcpy( address, pNext->address_1 );

		if( strlen( pNext->address_2 ) > 0 )
		{
			//
			// Use strcat to append one string to another
			//
			strcat( address, " " );
			strcat( address, pNext->address_2 );
		}

		//
		// Finally, format the city, state and zip code too.
		//
		sprintf( citystatezip, "%s, %s %s", pNext->city, pNext->state, pNext->zip );

		//
		// Now show all the fields using widths in
		// the format specifiers to align everything
		//
		// NOTE: Strings may be truncated
		//
		printf( "%3d %-20.20s %-20.20s %-15.15s %-16.16s\n",
			iRecord, name, address, citystatezip, pNext->phone );

		//
		// Move counter and pointer forward one
		//
		iRecord++;
		pNext++;
	}
	while( iRecord <= nRecords );

	//
	// Final pause
	//
	printf( "\nPress any key to continue..." );

	//
	// Since we don't need iPage anymore,
	// use it to hold the 'dummy' character
	//
	fflush(stdin);
	scanf( "%c", &iPage );
}

void PrintReport()
{
	CONTACT	*pNext = pRecords;		// Pointer for looping

	int		iRecord = 1,			// Counters
			iPage = 1;
	
	char	name[60],				// Temp String Buffers
			address[61],
			citystatezip[43];

	/****************************************************
	*  NOTE: On some systems, stdprn is not available,  *
	*        therefore, I am first print to a file and  *
	*        then issue a system command to actually    *
	*        print the report.  You may need to change  *
	*        the system call to match your operating    *
	*        system.  As rewritten this should work for *
	*        Windows.                                   *
	*****************************************************/

	FILE	*stdprn;

	//
	// Open the LPT1: port for writing in text mode
	//
	stdprn = fopen( "print.tmp", "w" );

	if( stdprn == NULL )
	{
		clearscreen();
		printf( "ERROR - Cannot temporary file for report!\n" );
		printf( "\nPress any key to continue..." );
		fflush(stdin);
		scanf( "%c", &iPage );
	}

	clearscreen();
	printf( "Printing" );

	//
	// Step 1: Prepare title of page
	//
	// Instructor's Note -
	//
	// This function is very similar to the DisplayRecords
	// function, except that we use fprintf to print to
	// the stdprn file (printer) instead of stdout (screen).
	//
	fprintf( stdprn, "                                CONTACTS             Page: %d\n\n", iPage );
	
	fprintf( stdprn, "### NAME                 ADDRESS              CITY, ST ZIP    PHONE NUMBER\n" );
	fprintf( stdprn, "--- -------------------- -------------------- --------------- ----------------\n" );


	//
	// Step 2: Loop through all records
	//
	do
	{
		//
		// Step 3: Eject a new page as needed
		//
		if( iRecord % 61 == 0 )		// Use % for determinine where to break
		{
			//
			// I use \f to send a form feed, then reprint the heading
			//
			fprintf( stdprn, "\f                                CONTACTS             Page: %d\n\n", iPage++ );
			
			fprintf( stdprn, "### NAME                 ADDRESS              CITY, ST ZIP    PHONE NUMBER\n" );
			fprintf( stdprn, "--- -------------------- -------------------- --------------- ----------------\n" );

		} // <----- Local variable 'key' drops out of scope here

		//
		// Use sprintf to 'format' to a string inside of stdout
		//
		sprintf( name, "%s, %s", pNext->last_name, pNext->first_name );

		//
		// Format the address also.
		//
		strcpy( address, pNext->address_1 );

		if( strlen( pNext->address_2 ) > 0 )
		{
			//
			// Use strcat to append one string to another
			//
			strcat( address, " " );
			strcat( address, pNext->address_2 );
		}

		//
		// Finally, format the city, state and zip code too.
		//
		sprintf( citystatezip, "%s, %s %s", pNext->city, pNext->state, pNext->zip );

		//
		// Now show all the fields using widths in
		// the format specifiers to align everything
		//
		// NOTE: Strings may be truncated
		//
		fprintf( stdprn, "%3d %-20.20s %-20.20s %-15.15s %-16.16s\n",
			iRecord, name, address, citystatezip, pNext->phone );

		//
		// Move counter and pointer forward one
		//
		iRecord++;
		pNext++;

		printf( "." );		// Print a dot for each record
	}
	while( iRecord <= nRecords );

	fclose(stdprn);

	//
	// Use this command to print (might need modification)
	//
	system( "NOTEPAD /p print.tmp" );

	//
	// Pause to let the user know we did something
	//
	printf( "\n\nCompleted.  Printed %d contacts.\n", nRecords );
	printf( "\nPress any key to continue..." );

	fflush(stdin);
	scanf( "%c", &iPage );
}

void SortRecords()
{
	int		sortby = 0;

	// Step 1: Find out which column the user wants to sort by
	clearscreen();

	printf( "\nAvailable sort sequences -\n\n" );
	printf( "1.  Name (Last, First)\n" );
	printf( "2.  City (City, Last Name, First Name)\n" );
	printf( "3.  State (State, City, Last Name, First Name)\n" );
	printf( "4.  Zip Code (Zip, Last Name, First Name)\n" );
	printf( "5.  Phone Number\n\n" );

	do
	{
		input_integer( "Enter desired sort sequence:", &sortby );

		if( sortby >= 1 && sortby <= 5 )
			break;

		if( sortby == 0 )
			return;

		printf( "\n\aInvalid choice, please try again.\n" );
	}
	while( 1 );

	// Step 2: Call qsort with the appropriate comparison function
	printf( "\nSorting..." );

	switch( sortby )
	{
	case 1:
		qsort( pRecords, nRecords, sizeof(CONTACT), sortname );
		break;
	case 2:
		qsort( pRecords, nRecords, sizeof(CONTACT), sortcity );
		break;
	case 3:
		qsort( pRecords, nRecords, sizeof(CONTACT), sortstate );
		break;
	case 4:
		qsort( pRecords, nRecords, sizeof(CONTACT), sortzip );
		break;
	case 5:
		qsort( pRecords, nRecords, sizeof(CONTACT), sortphone );
		break;
	}
}

void SearchRecords()
{
	int		sortby = 0;
	char	key[40];
	CONTACT	*pResult;

	// Step 1: Find out which column the user wants to search by
	clearscreen();

	printf( "\nAvailable search columns -\n\n" );
	printf( "1.  Last Name\n" );
	printf( "2.  City\n" );
	printf( "3.  State\n" );
	printf( "4.  Zip Code\n" );
	printf( "5.  Phone Number\n\n" );

	do
	{
		input_integer( "Enter desired search column:", &sortby );

		if( sortby >= 1 && sortby <= 5 )
			break;

		if( sortby == 0 )
			return;

		printf( "\n\aInvalid choice, please try again.\n" );
	}
	while( 1 );

	// Step 2: Let user enter value to search for
	printf( "\n" );
	key[0] = '\0';
	input_string( "Enter value to search for", key, 39 );

	if( strlen(key) == 0 )
		return;

	// Step 3: Sort the records, then do a search
	printf( "\nSearching..." );

	switch( sortby )
	{
	case 1:
		qsort( pRecords, nRecords, sizeof(CONTACT), sortname );
		pResult = bsearch( key, pRecords, nRecords, sizeof(CONTACT), searchname );
		break;
	case 2:
		qsort( pRecords, nRecords, sizeof(CONTACT), sortcity );
		pResult = bsearch( key, pRecords, nRecords, sizeof(CONTACT), searchcity );
		break;
	case 3:
		qsort( pRecords, nRecords, sizeof(CONTACT), sortstate );
		pResult = bsearch( key, pRecords, nRecords, sizeof(CONTACT), searchstate );
		break;
	case 4:
		qsort( pRecords, nRecords, sizeof(CONTACT), sortzip );
		pResult = bsearch( key, pRecords, nRecords, sizeof(CONTACT), searchzip );
		break;
	case 5:
		qsort( pRecords, nRecords, sizeof(CONTACT), sortphone );
		pResult = bsearch( key, pRecords, nRecords, sizeof(CONTACT), searchphone );
		break;
	}

	// Step 4: Pause and show results to user
	if( pResult != NULL )
	{
		printf( "\nFound It!\n" );
		printf( "Name: %s, %s\n", pResult->last_name, pResult->first_name );
		printf( "Address: %s\n", pResult->address_1 );
		printf( "City, State, Zip: %s, %s %s\n", pResult->city, pResult->state, pResult->zip );
		printf( "Phone Number: %s\n", pResult->phone );
	}
	else
	{
		printf( "\a\nNot Found!\n" );
	}

	printf( "\nPress enter to continue..." );
	fflush(stdin);
	scanf( "%c", &key[0] );
}

/*
   Menu
*/

int Menu()
{
	int	choice;

	// Step 1: Clear screen
	clearscreen();

	// Step 2: Draw menu
	puts( "     ***************************************   " );
	puts( "     *                                     *   " );
	puts( "     *     1.  Add new record              *   " );
	puts( "     *     2.  Edit a record               *   " );
	puts( "     *     3.  Delete a record             *   " );
	puts( "     *     4.  Display records             *   " );
	puts( "     *     5.  Print report                *   " );
	puts( "     *     6.  Sort records                *   " );
	puts( "     *     7.  Search                      *   " );
	puts( "     *     0.  Quit                        *   " );
	puts( "     *                                     *   " );
	puts( "     ***************************************   " );

	printf( "\n\n       Enter choice: " );

	// Step 3: Let user enter choice
	do
	{
		fflush(stdin);
		scanf( "%d", &choice );

		if( choice >= 0 && choice <= 7 )
			break;

		printf( "\a\nWrong!  Try again: " );
	}
	while( 1 );

	// Step 4: Return choice
	clearscreen();

	return choice;
}

void main()
{
	int	option;

	ReadRecords();

	do
	{
		option = Menu();

		switch( option )
		{
		case 0: 
			WriteRecords(); 
			printf( "Goodbye!\n" ); 
			exit(1); 
			break;
		case 1:
			AddRecord();
			break;
		case 2:
			EditRecord();
			break;
		case 3:
			DeleteRecord();
			break;
		case 4:
			DisplayRecords();
			break;
		case 5:
			PrintReport();
			break;
		case 6:
			SortRecords();
			break;
		case 7:
			SearchRecords();
			break;
		default: printf( "Unknown option!" );
		}

		/*
		if( repeat() == 'n' )
			break;
			*/
	}
	while( 1 );
}